import six

if six.PY2:
    from urlparse import urlparse
else:
    from urllib.parse import urlparse

from PIL import Image

from resizeimage import resizeimage

from .settings import DEFAULT_METHOD


def get_resize_arguments(**kwargs):
    width = kwargs.get('w')
    height = kwargs.get('h')
    method = kwargs.get('t')

    if not method:
        if width:
            method = DEFAULT_METHOD if height else 'width'
            size = [width, height] if height else width
        elif height:
            method = 'height'
            size = height
    else:
        size = [width, height]
    return method, size


def create_image(filename, new_filename, **kwargs):
    method, size = get_resize_arguments(**kwargs)
    resize_image(filename, new_filename, method, size)


def resize_image(filename, new_filename, method, size):
    """Resize an image according to a method and size and store it

    :param filename: file to resize
    :param new_filename: Name of the resized file
    :param method: resize method to use
    :param size:

    :raise IOError: Bad file or filename
    :raise ImageSizeError: Wrong image size
    """
    with open(filename, 'r+b') as f:
        with Image.open(f) as image:
            resized = resizeimage.resize(method, image, size)
            resized.save(new_filename, image.format)


def get_new_filename(filename, request):
    """There's no built-in to give path and parameters all together so
    do it manually."""
    return "{}?{}".format(filename, urlparse(request.url).query)



import os.path

from flask import Flask, abort, request, send_from_directory, jsonify
from webargs.flaskparser import use_kwargs
from resizeimage.resizeimage import ImageSizeError

from .fields import image_args, correct_arguments
from .settings import MEDIA_ROOT, MEDIA_URL
from .utils import create_image, get_new_filename

app = Flask(__name__)


@app.route(os.path.join('/', MEDIA_URL, "<path:filename>"))
@use_kwargs(image_args, validate=correct_arguments)
def get_image(filename, **kwargs):
    """Endpoint for getting image.

    URL is e.g. /MEDIA_URL/path/to/image.jpg
    for an image at MEDIA_ROOT/path/to/image.jpg
    plus query params e.g. ?w=100&h=100&t=cover
    If base image exists, check if params image exists and if not create
    the image and return it.
    """

    # Check if base image exists
    absolute_path = os.path.join(MEDIA_ROOT, filename)
    if not os.path.isfile(absolute_path):
        abort(404)

    # Get new filename based on request url
    resized_filename = get_new_filename(filename, request)
    resized_absolute_path = os.path.join(MEDIA_ROOT, resized_filename)

    # Check if image exists already
    if os.path.isfile(resized_absolute_path):
        return send_from_directory(MEDIA_ROOT, resized_filename)

    # Create and return new image
    try:
        create_image(absolute_path, resized_absolute_path, **kwargs)
        if os.path.isfile(resized_absolute_path):
            return send_from_directory(MEDIA_ROOT, resized_filename)
    except IOError as e:
        abort(400, "An error occured trying to open the file.")
    except ImageSizeError as e:
        abort(400, e.message)


@app.errorhandler(400)
def custom_400_handler(error):
    """Handle bad requests adding a message to the response."""
    return jsonify({
        'messages': getattr(error, 'description', 'Invalid Request'),
    }), 400


@app.errorhandler(422)
def custom_422_handler(error):
    """Handle unprocessable_entity requests adding an error to the response.

    Errors as thrown by  webargs library for invalid parameters.
    """
    exception = getattr(error, 'exc')
    return jsonify({
        'messages': exception.messages if exception else "Invalid Request"
    }), 422


if __name__ == "__main__":
    app.run()



from webargs import fields
from webargs.core import ValidationError

from . import settings


RESIZE_METHODS = ['contain', 'crop', 'cover', 'thumbnail']


def type_allowed(val):
    if val == settings.DEFAULT_METHOD:
        raise ValidationError(
            'Default type cannot be explicity chosen. Omit t parameter.',
            status_code=422)
    if val not in RESIZE_METHODS:
        raise ValidationError(
            'Resize type parameter not recognized',
            status_code=422)


def correct_arguments(args):
    if not args.get('w') and not args.get('h'):
        raise ValidationError(
            'Must provide at least one length parameter,w or h.',
            status_code=422)
    if args.get('t'):
        if not args.get('w') or not args.get('h'):
            raise ValidationError(
                'Must provide both w and h for this method.',
                status_code=422)


image_args = {
    'w': fields.Int(validate=lambda val: val > settings.MINIMUM_LENGTH),
    'h': fields.Int(validate=lambda val: val > settings.MINIMUM_LENGTH),
    't': fields.Str(validate=type_allowed)
}



import os
import os.path

MEDIA_ROOT = os.path.join('/', os.environ.get('ECCEHOMO_MEDIA_ROOT', ''))

MEDIA_URL = os.environ.get('ECCEHOMO_MEDIA_URL', '')

DEFAULT_METHOD = os.environ.get('ECCEHOMO_DEFAULT_METHOD', 'contain')

MINIMUM_LENGTH = int(os.environ.get('ECCEHOMO_MINIMUM_LEGNTH', 10))



# -*- coding: utf-8 -*-
#! /usr/bin/env python
import os
import subprocess
from setuptools import setup

import six

here = os.path.dirname(os.path.abspath(__file__))
README = open(os.path.join(here, 'README.rst')).read()
REQUIREMENTS = open(os.path.join(here, 'requirements/base.txt')).readlines()

def get_version_from_git_tag():
    command = "git describe --abbrev=0 --tags"
    if six.PY2:
        return subprocess.check_output(
            command, shell=True, stderr=subprocess.STDOUT).strip()

    return subprocess.getoutput(command).strip()

scripts = {
    "console_scripts": [
        "eccehomo=eccehomo.app:app",
    ]
}

setup(
    name='eccehomo',
    version=get_version_from_git_tag(),
    description='Micro application to serve images with customized cropping',
    long_description=README,
    author='DictGet Team',
    author_email='dictget@gmail.com',
    url="https://github.com/dictget/ecce-homo",
    download_url="https://github.com/dictget/ecce-homo/archive/{}.tar.gz".format(
        get_version_from_git_tag()
    ),
    packages=['eccehomo'],
    test_suite="eccehomo.tests",
    entry_points=scripts,
    install_requires=REQUIREMENTS,
    classifiers=[
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
)
